#include "SPI_drv.hpp"
#include <esp_log.h>
#include <cstring>
//#include <algorithm>

//#define DEBUG 1

namespace
{
    static constexpr char LOG_TAG[] = "SPI";
}

SPI_drv::SPI_drv() 
    : m_handle(nullptr)
{
}

void SPI_drv::init(spi_host_device_t host, int freq, gpio_num_t cs_pin, bool useMiso, uint8_t spiMode, uint32_t flags)
{
    int misoPin;
    if (useMiso) misoPin = (host == VSPI_HOST) ? 19 : 12;
    else misoPin = -1;

    spi_bus_config_t bus_config = {};
	bus_config.sclk_io_num     = (host == VSPI_HOST) ? 18 : 14;  // CLK
	bus_config.mosi_io_num     = (host == VSPI_HOST) ? 23 : 13; // MOSI
	bus_config.miso_io_num     = misoPin; // MISO
	bus_config.quadwp_io_num   = -1;      // Not used
	bus_config.quadhd_io_num   = -1;      // Not used
    bus_config.flags           = (SPICOMMON_BUSFLAG_SCLK | SPICOMMON_BUSFLAG_MOSI);

	esp_err_t errRc = spi_bus_initialize(
			host,
			&bus_config,
			host // DMA Channel
	);

	if (errRc != ESP_OK) {
		ESP_LOGE(LOG_TAG, "spi_bus_initialize(): rc=%d", errRc);
		abort();
	}

	spi_device_interface_config_t dev_config = {};

	
	dev_config.clock_speed_hz   = freq;
	dev_config.spics_io_num     = cs_pin;
	dev_config.flags            |= SPI_DEVICE_NO_DUMMY;
	dev_config.queue_size       = 1;
	/*добавляем фазу передачи команды длиной 8 бит */
	//dev_config.command_bits     = 8;
	//dev_config.input_delay_ns   = 10;
	dev_config.mode             = spiMode; //SPI mode, representing a pair of (CPOL, CPHA) configuration: 3 - (1, 1)
									 //																2 - (1, 0)
	dev_config.cs_ena_pretrans  = 0;
	dev_config.cs_ena_posttrans = 0;
	dev_config.duty_cycle_pos   = 128;
	//dev_config.flags           |= (SPI_DEVICE_BIT_LSBFIRST);
	dev_config.flags            |= flags;


	//ESP_LOGI(LOG_TAG, "... Adding device bus.");
	errRc = ::spi_bus_add_device(host, &dev_config, &m_handle);
	if (errRc != ESP_OK) {
		ESP_LOGE(LOG_TAG, "spi_bus_add_device(): rc=%d", errRc);
		abort();
	}
}

void SPI_drv::receive(std::uint8_t* rx_data, size_t dataLen)
{

	// spi_transaction_t t = {
    //     .cmd = CMD_READ | (addr & ADDR_MASK),
    //     .rxlength = 8,
    //     .flags = SPI_TRANS_USE_RXDATA,
    //     .user = ctx,
    // };

	//uint8_t buf[3] = {};

	assert(rx_data != nullptr);
	assert(dataLen > 0);

	spi_transaction_t trans_desc = {};
	trans_desc.length    = dataLen * 8;
	//trans_desc.tx_buffer = buf;
	//trans_desc.cmd = 0x00;
	trans_desc.rx_buffer = rx_data;
	//trans_desc.rxlength = 0;
    esp_err_t rc = spi_device_polling_transmit(m_handle, &trans_desc);
	//esp_err_t rc = spi_device_transmit(m_handle, &trans_desc);
	if (rc != ESP_OK) {
		ESP_LOGE(LOG_TAG, "transfer:spi_device_transmit: %d", rc);
	}	
}

void SPI_drv::transfer(std::uint8_t* data, size_t dataLen) {
	assert(data != nullptr);
	assert(dataLen > 0);

	spi_transaction_t trans_desc = {};
	trans_desc.length    = dataLen * 8;
	trans_desc.tx_buffer = data;
    esp_err_t rc = spi_device_polling_transmit(m_handle, &trans_desc);
	//esp_err_t rc = spi_device_transmit(m_handle, &trans_desc);
	if (rc != ESP_OK) {
		ESP_LOGE(LOG_TAG, "transfer:spi_device_transmit: %d", rc);
	}
} // transmit

void SPI_drv::cmd_write(std::uint16_t command, std::uint8_t* transfer_data, std::uint8_t byte2write){
	spi_transaction_t trans_desc = {};
	trans_desc.cmd      = command;
	trans_desc.length   = (byte2write)*8;
	trans_desc.tx_buffer = transfer_data;
	esp_err_t rc = spi_device_polling_transmit(m_handle, &trans_desc);
	//esp_err_t rc = spi_device_transmit(m_handle, &trans_desc);
	if (rc != ESP_OK) {
		ESP_LOGE(LOG_TAG, "transfer:spi_device_transmit: %d", rc);
	}
}

void SPI_drv::cmd_read(std::uint16_t command, std::uint8_t* read_data,std::uint8_t byte2read){
	spi_transaction_t trans_desc = {};
	trans_desc.length    =  byte2read*8;
	trans_desc.cmd       =  command;
	//trans_desc.rxlength  = 0;
	//trans_desc.flags     = SPI_TRANS_USE_RXDATA;
	trans_desc.rx_buffer = read_data;
	
	

    esp_err_t rc = spi_device_polling_transmit(m_handle, &trans_desc);
	//esp_err_t rc = spi_device_transmit(m_handle, &trans_desc);
	//*read_data = trans_desc.rx_data[0];
	if (rc != ESP_OK) {
		ESP_LOGE(LOG_TAG, "transfer:spi_device_transmit: %d", rc);
	}

}


uint8_t SPI_drv::transferByte(std::uint8_t value) {
	transfer(&value, 1);
	return value;
} // transferByte